@@ -196,7 +196,7 @@ class FromStatementParser:
             while next_token == ".":
                 right_hand, next_token = ExpressionParser.parse(
                     tokens,
-                    is_right_hand=True,
+                    is_chained_columns=True,
                 )
                 table_name = ChainedColumns(table_name, right_hand)
             expression = Table(table_name)
@@ -510,6 +510,7 @@ class ExpressionListParser:
         expression_tokens = []
         count_parenthesis = 0
         count_square_brackets = 0
+        in_string = False
         while next_token is not None:
             expression_tokens.append(next_token)
             if next_token == "(":
@@ -520,12 +521,15 @@ class ExpressionListParser:
                 count_square_brackets += 1
             elif next_token == "]":
                 count_square_brackets -= 1
+            elif next_token in String.QUOTES:
+                in_string = not in_string
 
             next_token = next(tokens, None)
             if next_token is None or (
                 next_token == ","
                 and count_parenthesis == 0
                 and count_square_brackets == 0
+                and in_string is False
             ):
                 expression, _ = ExpressionParser.parse(
                     iter(expression_tokens), can_be_type=can_be_type
@@ -545,6 +549,7 @@ class ExpressionParser:
         can_alias=True,
         until_one_of=None,
         first_token=None,
+        is_chained_columns=False,
     ) -> Tuple[Expression, Any]:
         until_one_of = until_one_of or []
 
@@ -793,28 +798,31 @@ class ExpressionParser:
             )
             next_token = next(tokens, None)
 
-        if next_token and next_token in ("+", "-", "*", "/"):
+        while next_token == ".":
+            right_hand, next_token = ExpressionParser.parse(
+                tokens, until_one_of=until_one_of, is_chained_columns=True
+            )
+            expression = ChainedColumns(expression, right_hand)
+
+        if next_token and next_token in ("+", "-", "*", "/") and not is_chained_columns:
             left_hand = expression
             symbol = next_token
             right_hand, next_token = ExpressionParser.parse(
-                tokens, is_right_hand=True, until_one_of=until_one_of
+                tokens,
+                is_right_hand=True,
+                until_one_of=until_one_of,
             )
             expression = ArithmaticOperator(symbol, left_hand, right_hand)
 
-        if next_token == ".":
-            right_hand, next_token = ExpressionParser.parse(
-                tokens, is_right_hand=True, until_one_of=until_one_of
+        while next_token == "[":
+            argument_tokens, next_token = get_tokens_until_one_of(
+                tokens, stop_words=["]"]
             )
-            expression = ChainedColumns(expression, right_hand)
-            if next_token is not None and next_token == "[":
-                argument_tokens, next_token = get_tokens_until_one_of(
-                    tokens, stop_words=["]"]
-                )
-                arguments = ExpressionListParser.parse(iter(argument_tokens))
-                expression = Index(expression, arguments)
-                next_token = next(tokens, None)
+            arguments = ExpressionListParser.parse(iter(argument_tokens))
+            expression = Index(expression, arguments)
+            next_token = next(tokens, None)
 
-        if is_right_hand:
+        if is_right_hand or is_chained_columns:
             return expression, next_token
 
         if lower(next_token) in Condition.PREDICATES:
