@@ -1,3 +1,4 @@
+from dataclasses import dataclass
 from typing import Any, List, Optional, Set
 
 from sqlvalidator.grammar.tokeniser import lower
@@ -11,6 +12,15 @@ def transform(obj: Any) -> str:
     return str(obj)
 
 
+@dataclass
+class _FieldInfo:
+    name: str
+    type: type
+
+    def __hash__(self):
+        return hash(self.name)
+
+
 class SelectStatement:
     def __init__(
         self,
@@ -113,19 +123,16 @@ class SelectStatement:
             statement_str += ";"
         return statement_str
 
-    def validate(self, known_fields: Optional[Set[str]] = None) -> list:
+    def validate(self, known_fields: Optional[Set[_FieldInfo]] = None) -> list:
         errors = []
         known_fields = known_fields or set()
-
-        if isinstance(self.from_statement, Parenthesis) and isinstance(
-            self.from_statement.args[0], SelectStatement
-        ):
-            known_fields = known_fields | self.from_statement.args[0].known_fields
-        elif isinstance(self.from_statement, Table):
-            known_fields = known_fields | {"*"}
+        if hasattr(self.from_statement, "known_fields"):
+            known_fields = known_fields | self.from_statement.known_fields
 
         for e in self.expressions:
             errors += e.validate(known_fields)
+        if self.from_statement:
+            errors += self.from_statement.validate(known_fields=set())
         if self.where_clause:
             errors += self.where_clause.validate(known_fields)
         if self.group_by_clause:
@@ -141,16 +148,28 @@ class SelectStatement:
         return errors
 
     @property
-    def known_fields(self) -> Set[str]:
+    def known_fields(self) -> Set[_FieldInfo]:
         fields = set()
         for e in self.expressions:
             if isinstance(e, Column):
-                fields.add(e.value)
+                fields.add(_FieldInfo(e.value, e.return_type))
             elif isinstance(e, Alias):
                 if isinstance(e.alias, Column):
-                    fields.add(e.alias.value)
+                    fields.add(_FieldInfo(e.alias.value, e.return_type))
                 else:
-                    fields.add(e.alias)
+                    fields.add(_FieldInfo(e.alias, e.return_type))
+
+        if self.from_statement:
+            if isinstance(self.from_statement, Parenthesis) and isinstance(
+                self.from_statement.args[0], SelectStatement
+            ):
+                subquery_known_fields = self.from_statement.args[0].known_fields
+                if any(f.name == "*" for f in fields) and not any(
+                    f.name == "*" for f in subquery_known_fields
+                ):
+                    fields = {f for f in fields if f.name != "*"}
+                    fields |= subquery_known_fields
+
         return fields
 
     def __eq__(self, other):
@@ -225,12 +244,18 @@ class Expression:
     def __eq__(self, other):
         return type(self) == type(other) and self.value == other.value
 
-    def validate(self, known_fields: Set[str]) -> list:
+    def validate(self, known_fields: Set[_FieldInfo]) -> list:
         return []
 
     @property
     def return_type(self):
-        return Any
+        return object
+
+    def resolve_return_type(self, known_fields):
+        matching_fields = [f for f in known_fields if f.name == transform(self)]
+        if matching_fields:
+            return matching_fields[0].type
+        return self.return_type
 
 
 class WhereClause(Expression):
@@ -242,13 +267,14 @@ class WhereClause(Expression):
             return "\n " + transformed_value.replace("\n", "\n ")
         return " " + transformed_value
 
-    def validate(self, known_fields: Set[str]) -> list:
+    def validate(self, known_fields: Set[_FieldInfo]) -> list:
         errors = super().validate(known_fields)
         errors += self.value.validate(known_fields)
-        if self.value.return_type != bool:
+        value_type = self.value.resolve_return_type(known_fields)
+        if value_type not in (bool, object):
             errors.append(
                 "The argument of WHERE must be type boolean, not type {}".format(
-                    self.value.return_type
+                    value_type.__name__
                 )
             )
         return errors
@@ -302,13 +328,19 @@ class GroupByClause(Expression):
             elif (
                 isinstance(arg, (Column, String))
                 and (
-                    arg.value not in known_fields
+                    not any(f.name == arg.value for f in known_fields)
                     and arg.value
                     not in [e.alias for e in select_expressions if isinstance(e, Alias)]
                 )
-                and "*" not in known_fields
+                and not any(f.name == "*" for f in known_fields)
             ):
-                errors.append('column "{}" does not exist'.format(arg.value))
+                fields_without_alias = [
+                    f for f in known_fields if f.name.split(".", 1)[-1] == arg.value
+                ]
+                if len(fields_without_alias) > 1:
+                    errors.append('column "{}" is ambiguous'.format(self.value))
+                elif len(fields_without_alias) == 0:
+                    errors.append('column "{}" does not exist'.format(arg.value))
 
         return errors
 
@@ -325,10 +357,11 @@ class HavingClause(Expression):
     def validate(self, known_fields):
         errors = super().validate(known_fields)
         errors += self.value.validate(known_fields)
-        if self.value.return_type != bool:
+        value_type = self.value.resolve_return_type(known_fields)
+        if value_type not in (bool, object):
             errors.append(
-                "The argument of WHERE must be type boolean, not type {}".format(
-                    self.value.return_type
+                "The argument of HAVING must be type boolean, not type {}".format(
+                    value_type.__name__
                 )
             )
         return errors
@@ -518,6 +551,12 @@ class FunctionCall(Expression):
             function_str += ", ".join(transformed_args) + ")"
         return function_str
 
+    def validate(self, known_fields):
+        errors = super().validate(known_fields)
+        for a in self.args:
+            errors += a.validate(known_fields)
+        return errors
+
     def __repr__(self):
         return "<FunctionCall {} - {}>".format(
             self.function_name, ", ".join(map(repr, self.args))
@@ -546,14 +585,14 @@ class CastFunctionCall(FunctionCall):
 
 class CountFunctionCall(FunctionCall):
     def __init__(self, *args, distinct=False):
-        super().__init__("count", args)
+        super().__init__("count", *args)
         self.distinct = distinct
 
     def __str__(self):
         return "{}({}{})".format(
             self.function_name.upper(),
             "DISTINCT " if self.distinct else "",
-            ", ".join(map(transform, *self.args)),
+            ", ".join(map(transform, self.args)),
         )
 
     def __eq__(self, other):
@@ -618,6 +657,12 @@ class FilteredFunctionCall(Expression):
             and self.filter_condition == other.filter_condition
         )
 
+    def validate(self, known_fields):
+        errors = super().validate(known_fields)
+        errors += self.function_call.validate(known_fields)
+        errors += self.filter_condition.validate(known_fields)
+        return errors
+
 
 class AnalyticsClause(Expression):
     def __init__(self, function, partition_by, order_by, frame_clause):
@@ -679,6 +724,16 @@ class AnalyticsClause(Expression):
             self.frame_clause,
         )
 
+    def validate(self, known_fields):
+        errors = super().validate(known_fields)
+        errors += self.function.validate(known_fields)
+        if self.partition_by:
+            for partition in self.partition_by:
+                errors += partition.validate(known_fields)
+        if self.order_by:
+            errors += self.order_by.validate(known_fields, None)
+        return errors
+
 
 class WindowFrameClause(Expression):
     def __init__(self, rows_range, frame):
@@ -730,8 +785,18 @@ class Column(Expression):
 
     def validate(self, known_fields):
         errors = super().validate(known_fields)
-        if self.value not in known_fields and "*" not in known_fields:
-            errors.append("The column {} was not found".format(self.value))
+        if (
+            not any(f.name == self.value for f in known_fields)
+            and self.value != "*"
+            and not any(f.name == "*" for f in known_fields)
+        ):
+            fields_without_alias = [
+                f for f in known_fields if f.name.split(".", 1)[-1] == self.value
+            ]
+            if len(fields_without_alias) > 1:
+                errors.append('column "{}" is ambiguous'.format(self.value))
+            elif len(fields_without_alias) == 0:
+                errors.append("The column {} was not found".format(self.value))
         return errors
 
 
@@ -754,7 +819,19 @@ class ChainedColumns(Expression):
 
     @property
     def return_type(self):
-        return self.columns[0].return_type
+        return self.columns[-1].return_type
+
+    def validate(self, known_fields):
+        errors = super().validate(known_fields)
+        full_value = str(self)
+        alias = ".".join(map(transform, self.columns[:-1]))
+        last_value = self.columns[-1]
+        if (
+            not any(f.name == full_value for f in known_fields)
+            and not any(f.name == f"{alias}.*" for f in known_fields)
+        ) and not any(f.name == "*" for f in known_fields):
+            errors.append(f"The column {last_value} was not found in alias {alias}")
+        return errors
 
 
 class Type(Expression):
@@ -926,6 +1003,12 @@ class Parenthesis(Expression):
             errors += a.validate(known_fields)
         return errors
 
+    @property
+    def known_fields(self) -> Set[_FieldInfo]:
+        if hasattr(self.args[0], "known_fields"):
+            return self.args[0].known_fields
+        return set()
+
     @property
     def return_type(self):
         for a in self.args:
@@ -978,14 +1061,20 @@ class Alias(Expression):
 
     def validate(self, known_fields):
         errors = super().validate(known_fields)
-        if isinstance(self.expression, Column):
-            errors += self.expression.validate(known_fields)
+        errors += self.expression.validate(known_fields)
         return errors
 
     @property
     def return_type(self):
         return self.expression.return_type
 
+    @property
+    def known_fields(self) -> Set[_FieldInfo]:
+        return {
+            _FieldInfo("{}.{}".format(transform(self.alias), f.name), f.type)
+            for f in self.expression.known_fields
+        }
+
 
 class Index(Expression):
     def __init__(self, expression, indices):
@@ -1067,6 +1156,10 @@ class Table(Expression):
             table_str = f"[{table_str}]"
         return table_str
 
+    @property
+    def known_fields(self) -> Set[_FieldInfo]:
+        return {_FieldInfo("*", type=object)}
+
 
 class Unnest(Expression):
     def __init__(self, unnest_expression, with_offset, with_offset_as, offset_alias):
@@ -1212,12 +1305,36 @@ class Join(Expression):
             self.using,
         )
 
-    def validate(self, known_fields: Set[str]) -> list:
+    def validate(self, known_fields: Set[_FieldInfo]) -> list:
         errors = super().validate(known_fields)
         if self.join_type not in ("CROSS JOIN", ",") and not (self.using or self.on):
             errors.append("Missing ON or USING for join")
         return errors
 
+    @property
+    def known_fields(self) -> Set[_FieldInfo]:
+        known_fields = set()
+        if isinstance(self.left_from, Alias):
+            left_alias = self.left_from.alias
+            for field in self.left_from.expression.known_fields:
+                known_fields.add(_FieldInfo(left_alias + "." + field.name, field.type))
+        else:
+            known_fields |= self.left_from.known_fields
+
+        if isinstance(self.right_from, Alias):
+            right_alias = self.right_from.alias
+            for field in self.right_from.expression.known_fields:
+                known_fields.add(
+                    _FieldInfo(
+                        right_alias + "." + field.name,
+                        field.type,
+                    )
+                )
+        else:
+            known_fields |= self.right_from.known_fields
+
+        return known_fields
+
 
 class CombinedQueries(Expression):
     SET_OPERATORS = ("union", "intersect", "except", "all")
@@ -1320,7 +1437,10 @@ class Condition(Expression):
     def validate(self, known_fields):
         errors = super().validate(known_fields)
         errors += self.value.validate(known_fields)
-        errors += self.right_hand.validate(known_fields)
+        if self.predicate.lower() == "between":
+            errors += self.right_hand.validate(known_fields, is_between_predicate=True)
+        else:
+            errors += self.right_hand.validate(known_fields)
         return errors
 
     @property
@@ -1372,16 +1492,18 @@ class BooleanCondition(Expression):
             and all(a == o for a, o in zip(self.args, other.args))
         )
 
-    def validate(self, known_fields):
+    def validate(self, known_fields, is_between_predicate=False):
         errors = super().validate(known_fields)
         for a in self.args:
             errors += a.validate(known_fields)
-            if a.return_type != bool and a.return_type != Any:
-                errors.append(
-                    "The argument of {} must be type boolean, not type {}".format(
-                        self.type.upper(), a.return_type
+            if not is_between_predicate:
+                a_type = a.resolve_return_type(known_fields)
+                if a_type not in (bool, object):
+                    errors.append(
+                        "The argument of {} must be type boolean, not type {}".format(
+                            self.type.upper(), a_type.__name__
+                        )
                     )
-                )
 
         return errors
 
